<?php
namespace Planejamento\Logic;

use Geral\Entities\Servico;
use \Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Geral\Entities\Paramgeral;
use \Faturamento\Entities\Efetclie;
use \Planejamento\Entities\Cardadia;
use \Planejamento\Entities\Compcard;
use \Planejamento\Entities\Paramplaneja;
use \Indeterminado\Entities\Servifat;
use \Indeterminado\Entities\Logaltefet;
use \Planejamento\Repositories\Retirada;
use \Planejamento\Repositories\Itlanctoest;
use \Planejamento\Repositories\Itemretiprat;

class EfetivosPlanejamento
{

    public static $mensagem_retorno = array('MSG', 'PARAMS');
    const MSG_QTD_INFORMADA_CLIENTES = 'pla01000_qtd_informada_cliente';
    const MSG_QTD_INFORMADA_FUNC     = 'pla01000_qtd_informada_funcionario';
    const MSG_QTD_INFORMADA_DOACOES  = 'pla01000_qtd_informada_doacao';
    const MSG_QTD_INFORMADA_EFETIVOS = 'pla01000_qtd_informada_efetivos';
    const MSG_QTD_INFORMADA_PRATO    = 'pla_qtd_informada_prato';
    const MSG_QTD_VENDIDA            = 'pla_qtd_vendida';

    const MSG_DTCARDAPIO_DTPROCESSAMENTO = 'pla_dtcardapio_menor_igual_dtprocessamento';

    const MSG_PRODUTO_BAIXADO_ESTOQUE = 'pla_produto_baixado_estoque';

    const MSG_VALIDA_CARDAPIO_COM_RETIRADA = 'pla_valida_cardapio_com_retirada';

    const MSG_BLOQUEIA_EFETIVOS_DIAS_FECHADOS = 'pla_bloqueia_efetivos_dias_fechados';

    const MSG_SOMATORIO_CLIENTES_IGUAL     = 'pla01000_somatorio_clientes_igual';
    const MSG_SOMATORIO_CLIENTES_DIFERENTE = 'pla01000_somatorio_clientes_diferente';



    public static function enviaMensagemRetorno($mensagem, array $parametros=null)
    {
        return self::$mensagem_retorno = array('MSG' => $mensagem, 'PARAMS' => $parametros);
    }


    /**
     * Soma a quantidade de comensais para clientes, funcionarios e doacoes em comensais efetivos
     *
     * @static
     * @param integer $qtd_clientes     Qtde de clientes efetivos
     * @param integer $qtd_funcionarios Qtde de funcionarios efetivos
     * @param integer $qtd_doacoes      Qtde de doacoes efetivos
     * @return mixed
     */
    //@todo O campo “Efetivo” exibe o somatório das quantidades informadas nos três campos
    public static function somaComensaisEfetivos($qtd_clientes, $qtd_funcionarios, $qtd_doacoes)
    {
        return $qtd_clientes + $qtd_funcionarios + $qtd_doacoes;
    }


    /**
     * Valida se a quantidade informada de efetivos e maior ou igual a zero
     *
     * @static
     * @param  integer $quantidade  Quantidade informada
     * @param  string  $item        Item informado
     * @return string               Mensagem de retorno
     */
    //@todo acao deve ser realizada somente sobre o registro que foi alterado
    //@todo variavel $item sera identificada na camada facade com o nome do field
    public static function verificaQuantidadeInformada($quantidade, $item)
    {
        if($quantidade < 0){
            switch($item){
                case "CLIENTES": return self::enviaMensagemRetorno(self::MSG_QTD_INFORMADA_CLIENTES);
                case "FUNCIONARIOS": return self::enviaMensagemRetorno(self::MSG_QTD_INFORMADA_FUNC);
                case "DOACOES": return self::enviaMensagemRetorno(self::MSG_QTD_INFORMADA_DOACOES);
                case "COMENSAIS": return self::enviaMensagemRetorno(self::MSG_QTD_INFORMADA_EFETIVOS);
                case "PRATO": return self::enviaMensagemRetorno(self::MSG_QTD_INFORMADA_PRATO);
                case "QTD_VENDIDA": return self::enviaMensagemRetorno(self::MSG_QTD_VENDIDA);
            }
        }
    }

    /**
     * Valida permissao para lancar efetivos com data do cardapio superior a data de processamento
     *
     * @static
     * @param integer    $nrorg       Numero da organizacao
     * @param integer    $cdfilial    Codigo da filial
     * @param integer    $dtCardapio  Data do cardapio
     * @return string                 Mensagem de retorno
     */
    //@todo acao deve ser realizada ao tentar editar qualquer informacao de comensal efetivo
    //@todo centralizar consultas da Paramplaneja depois que houver refatoracao no arquivo de Parametros ou Ambiente
    public static function verificaPermissaoLancamentoEfetivosDataSuperior($nrorg, $cdfilial, $dtCardapio)
    {
        $dtCardapio = new \TkDateTime($dtCardapio, \TkDateTime::FORMAT_DATE);
        $dtProcessamento = new \TkDateTime(\ParametrosEstoque::getDataProcessamento(), \TkDateTime::FORMAT_DATE);
        if ($dtCardapio->isLater($dtProcessamento)) {
            /** @var $paramPlaneja Paramplaneja */
            $paramPlaneja = Paramplaneja::getObject(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'cdparplaneja' => 1));
            $idPermissaoLancamentoDataSuperior = $paramPlaneja->getIdperefeposdtpr();
            if ($idPermissaoLancamentoDataSuperior == 'N') {
                return self::enviaMensagemRetorno(self::MSG_DTCARDAPIO_DTPROCESSAMENTO);
            }
        }

    }


    /**
     * Valida permissao para lancar os efetivos sem retirada do planejamento
     *
     * @static
     * @param integer $nrorg      Numero da organizacao
     * @param integer $cdfilial   Codigo da filial
     * @param integer $nrcardapio Numero do cardapio
     * @return string             Mensagem de retorno
     */
    //@todo deve ser acionada antes da edicao das informacoes de comensais por cardapio, prato e/ou comensais por cliente.
    public static function verificaPermissaoLancamentoSemRetirada($nrorg, $cdfilial, $nrcardapio)
    {
        /** @var $paramPlaneja Paramplaneja */
        $paramPlaneja = Paramplaneja::getObject(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'cdparplaneja' => 1));
        $idPermissaoLancamentoSemRetirada = $paramPlaneja->getIdperlefesretpla();
        if ($idPermissaoLancamentoSemRetirada == 'N') {
            $qtdItens = Itlanctoest::QtdRetiradasEvolucaoEstoque($nrorg, $cdfilial, $nrcardapio);
            if ($qtdItens == 0) {
                return self::enviaMensagemRetorno(self::MSG_PRODUTO_BAIXADO_ESTOQUE);
            }
        }
    }


    /**
     * Verifica se a quantidade informada de comensais do cliente (Efetivos do Planejamento) é igual ao somatório de comensais
     * informados por cliente em “Efetivos de Faturamento”. Se houver utilizacao da consistencia sobre o minimo de efetivos,
     * havera bloqueio para o usuario, caso nao utilize a consistencia apenas informa o usuario da diferença encontrada de comensais para cliente.
     * @static
     * @param integer $nrorg      Numero da organizacao
     * @param integer $cdfilial   Codigo da filial
     * @param integer $cdservico  Codigo do servico
     * @param integer $nrcardapio Numero do cardapio
     * @param integer $dtefetclie Data do cardapio
     * @return string             Mensagem de retorno
     */
    //@todo deve ser acionada na saída da tela de Efetivos, na mudança de abas (efetivos do planejamento ou efetivos do faturamento) ou atraves do acionamento de uma action destinada a esta finalidade.
    public static function utilizaConsistenciaSobreMinimoEfetivos($nrorg, $cdfilial, $cdservico, $nrcardapio, $dtefetclie)
    {
        $dtefetclie = new \DateTime($dtefetclie);

        /** @var $paramGeral Paramgeral */
        $paramGeral = Paramgeral::getObjectByWhere("NRORG = ?", array($nrorg));
        $idUtilizaConsistenciaMinEfetivos = $paramGeral->getIdconsminefe();

        /** @var $cardaDia Cardadia */
        $cardaDia = Cardadia::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND NRCARDAPIO = ?", array($nrorg, $cdfilial, $nrcardapio));
        $qtdComensaisClienteEfetivos = $cardaDia->getQtcliefetcar();

        $efetClie = Efetclie::getObjectsByWhere("NRORG = ? AND CDFILIAL = ? AND CDSERVPLAN = ? AND DTEFETCLIE = ?",
            array($nrorg, $cdfilial, $cdservico, $dtefetclie));

        $somaComensaisFaturados = 0;
        $nmclientes = '';
        $nmservico = '';

        /** @var $qtdComensaisCliente Efetclie */
        foreach ($efetClie as $qtdComensaisCliente ) {
            $somaComensaisFaturados += $qtdComensaisCliente->getQtcomefetcli();
            $nmclientes .= $qtdComensaisCliente->getCliente()->getNmrazsocclie().'<br>';
            $nmservico = $qtdComensaisCliente->getServplan()->getNmservico();
        }

        if (!empty($efetClie)) {
            if ($idUtilizaConsistenciaMinEfetivos == 'S') {
                if ($qtdComensaisClienteEfetivos != $somaComensaisFaturados) {
                    //@todo A mensagem supracitada deve bloquear a saida da tela.
                    return self::enviaMensagemRetorno(self::MSG_SOMATORIO_CLIENTES_IGUAL, array('CLIENTE' => $nmclientes, 'SERVICO' => $nmservico));
                }
            } else {
                if ($qtdComensaisClienteEfetivos != $somaComensaisFaturados) {
                    //@todo Nesse caso, somente exibir uma mensagem informativa.
                    return self::enviaMensagemRetorno(self::MSG_SOMATORIO_CLIENTES_DIFERENTE, array('CLIENTE' => $nmclientes, 'SERVICO' => $nmservico));
                }
            }
        }

    }


    /**
     * Obriga que um cardapio que possui retirada tenha um numero de efetivos superior a zero.
     * @static
     * @param integer $nrorg        Numero da organizacao
     * @param integer $cdfilial     Codigo da filial
     * @param integer $nrcardapio   Numero do cardapio
     * @param integer $funcionarios Quantidade de funcionarios
     * @param integer $doacoes      Quantidade de doacoes
     * @param integer $clientes     Quantidade de clientes
     * @return string               Mensagem de retorno
     */
    //@todo Esta validação deve ser acionada antes da confirmação de alterações realizadas sobre os efetivos do cardápio.
    //@todo variaveis $funcionarios, $doacoes, $clientes serao identificadas na camada facade com o nome do field
    public static function validaCardapioComRetirada($nrorg, $cdfilial, $nrcardapio, $funcionarios, $doacoes, $clientes)
    {
        if ($funcionarios == 0 || $doacoes == 0 || $clientes == 0) {
            $retirada = Retirada::qry_QtdRetiradasIndependenteEvolucaoEstoque($nrorg, $cdfilial, $nrcardapio);
            if ($retirada > 0) {
                return self::enviaMensagemRetorno(self::MSG_VALIDA_CARDAPIO_COM_RETIRADA);
            }
        }
    }


    /**
     * Valida se o intervalo entre a data do cardapio e a data de processamento da filial ultrapassa a tolerancia de dias
     * estabelecidos para lancamento dos efetivos
     * @static
     * @param integer     $nrorg      Numero da organizacao
     * @param integer     $cdfilial   Codigo da filial
     * @param integer     $dtcardapio Data do cardapio
     * @return string                 Mensagem de retorno
     */
    //@todo deve ser acionada ao tentar editar qualquer informação de comensal efetivo
    //@todo centralizar consultas da Paramplaneja depois que houver refatoracao no arquivo de Parametros ou Ambiente
    public static function utilizaBloqueioEfetivosParaDiasFechados($nrorg, $cdfilial, $dtcardapio)
    {
        $dtcardapio = new \TkDateTime($dtcardapio);
        /** @var $paramPlaneja Paramplaneja */
        $paramPlaneja = Paramplaneja::getObject(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'cdparplaneja' => 1));
        $idBloqueiaAlteracaoDiasFechados = $paramPlaneja->getIdblqefediafech();

        if ($idBloqueiaAlteracaoDiasFechados == 'S'){
            $dtProcessamento = new \TkDateTime(\ParametrosEstoque::getDataProcessamento(), \TkDateTime::FORMAT_DATE);
            $intervalo_dias = count($dtProcessamento->intervalToArray($dtcardapio, \TkDateTime::INTERVAL_DAYS));
            $nrDiasAnterioresDtProc = $paramPlaneja->getNrdiaantdtproc();
            if ($intervalo_dias > $nrDiasAnterioresDtProc) {
                return self::enviaMensagemRetorno(self::MSG_BLOQUEIA_EFETIVOS_DIAS_FECHADOS, array('DIAS_ANTERIORES' => $nrDiasAnterioresDtProc));
            }
        }
    }


    /**
     * Transacao acionada ao atualizar os comensais efetivos
     * @static
     * @param integer     $nrorg               Numero da organizacao
     * @param integer     $cdfilial            Codigo da filial
     * @param integer     $cdservico           Codigo do servico
     * @param integer     $nrcardapio          Numero do cardapio
     * @param integer     $dtcardapio          Data do cardapio
     * @param integer     $qtd_comensais_ant   Quantidade de comensais anterior
     * @param integer     $qtd_comensais_atual Quantidade de comensais atual
     */
    //@todo deve ser acionado ao atualizar a quantidade de comensais efetivos
    public static function atualizaEfetivosPlanejamento(
        $nrorg, $cdfilial, $cdservico, $nrcardapio, $dtcardapio, $qtd_comensais_ant, $qtd_comensais_atual, $operador
    )
    {
        try {
            \Setforms2\Runtime\Registry::getEntityManager()->beginTransaction();
            self::atualizaQtdComensaisPorPrato($nrorg, $cdfilial, $nrcardapio, $qtd_comensais_atual);
            self::atualizaQtdProduzidaPorPrato($nrorg, $cdfilial, $nrcardapio);
            self::insereLogAlteracaoEfetivos($nrorg, $cdfilial, $cdservico, $dtcardapio, $qtd_comensais_ant, $qtd_comensais_atual, $operador);
            \Setforms2\Runtime\Registry::getEntityManager()->flush();
            \Setforms2\Runtime\Registry::getEntityManager()->commit();
        } catch (\Exception $e) {
            \Setforms2\Runtime\Registry::getEntityManager()->rollback();
            tk_warning($e->getMessage());
        }
    }


    /**
     * Transacao acionada ao atualizar os clientes efetivos
     *
     * @static
     * @param integer     $nrorg               Numero da organizacao
     * @param integer     $cdfilial            Codigo da filial
     * @param integer     $cdservico           Codigo do servico
     * @param integer     $dtcardapio          Data do cardapio
     * @param integer     $qtcomeefetclie      Qtde de comensais do cliente
     */
    //@todo deve ser acionado ao atualizar a quantidade de comensais efetivos do cliente
    public static function atualizaClientesEfetivosFaturamento($nrorg, $cdfilial, $cdservico, $dtcardapio, $qtcomeefetclie)
    {
        try {
            \Setforms2\Runtime\Registry::getEntityManager()->beginTransaction();
            /** @var $servicoFaturado Servifat */
            $servicoFaturado = Servifat::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND NVL(CDSERVICFATURAM, CDSERVFILI) = ?",
                                                            array($nrorg, $cdfilial, $cdservico));
            $clienteFaturado = \Planejamento\Repositories\Efetclie::ClienteEfetivosFaturamento($nrorg, $cdfilial, $cdservico, $dtcardapio);
            if (is_object($servicoFaturado) && count($clienteFaturado) == 1) {
                /** @var $efetClie Efetclie */
                $efetClie = Efetclie::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ? AND DTEFETCLIE = ? AND CDCLIENTE = ? AND CDSERVPLAN = ?",
                                                        array($nrorg, $cdfilial, $cdservico, $dtcardapio, $clienteFaturado[0]['CDCLIENTE'], $cdservico));
                if (is_object($efetClie)) {
                    $efetClie->getQtcomefetcli($qtcomeefetclie);
                } else {
                    $efetClie = new Efetclie();
                    $dtcardapio = new \DateTime($dtcardapio);
                    /** @var $Cliente \Geral\Entities\Base\Cliente */
                    $Cliente = \Geral\Entities\Base\Cliente::getObjectByWhere("NRORG = ? AND CDCLIENTE = ?", array($nrorg, $clienteFaturado[0]['CDCLIENTE']));
                    $servfat = ($servicoFaturado->getServicfaturam()->getCdservfili()) ?
                        $servicoFaturado->getServicfaturam()->getCdservfili() : $servicoFaturado->getServfili()->getCdservfili();
                    /** @var $Servico \Geral\Entities\Base\Servico */
                    $Servico = \Geral\Entities\Base\Servico::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ?", array($nrorg, $cdfilial, $servfat));
                    /** @var $ServicoPlan \Geral\Entities\Base\Servico */
                    $ServicoPlan = \Geral\Entities\Base\Servico::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ?", array($nrorg, $cdfilial, $cdservico));
                    $efetClie->setCdfilial($cdfilial);
                    $efetClie->setServfili($Servico);
                    $efetClie->setDtefetclie($dtcardapio);
                    $efetClie->setCliente($Cliente);
                    $efetClie->setServplan($ServicoPlan);
                    $efetClie->setQtcomefetcli($qtcomeefetclie);
                    $efetClie->setVrservfatuefcli(null);
                    $efetClie->setQtvendaefetcli(null);
                }
                $efetClie->persist();
                \Setforms2\Runtime\Registry::getEntityManager()->flush();
                \Setforms2\Runtime\Registry::getEntityManager()->commit();
            }
        } catch (\Exception $e) {
            \Setforms2\Runtime\Registry::getEntityManager()->rollback();
            tk_warning($e->getMessage());
        }
    }


    /**
     * Atualiza qtde de comensais por prato
     * @static
     * @param integer     $nrorg          Numero da organizacao
     * @param integer     $cdfilial       Codigo da filial
     * @param integer     $nrcardapio     Numero do cardapio
     * @param integer     $qtd_comensais  Quantidade de comensais
     */
    private static function atualizaQtdComensaisPorPrato($nrorg, $cdfilial, $nrcardapio, $qtd_comensais)
    {
        $compCard = Compcard::getObjectsByWhere("NRORG = ? AND CDFILIAL = ? AND NRCARDAPIO = ?", array($nrorg, $cdfilial, $nrcardapio));
        /** @var $prato Compcard */
        foreach ($compCard as $prato) {
            $perc_consumo = ($qtd_comensais * $prato->getVrpeconscard())/100;
            $prato->setQtcomefetpra($perc_consumo);
            $prato->persist();
        }
    }


    /**
     * Atualiza qtde produzida por prato
     * @static
     * @param integer     $nrorg          Numero da organizacao
     * @param integer     $cdfilial       Codigo da filial
     * @param integer     $nrcardapio     Numero do cardapio
     */
    private static function atualizaQtdProduzidaPorPrato($nrorg, $cdfilial, $nrcardapio)
    {
        $pratos = Itemretiprat::qry_QtdProduzidaPorPrato($nrorg, $cdfilial, $nrcardapio);
        foreach ($pratos as $prato){
            /** @var $compCard Compcard */
            $compCard = Compcard::getObject(array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrcardapio' => $nrcardapio,
            'cdtipoprat' => $prato['CDTIPOPRAT'], 'cdfilauxprat' => $prato['CDFILAUXPRAT'], 'cdprato' => $prato['CDPRATO']));
            $compCard->setQtproduzprat($prato['QTPRODUZIDA']);
            $compCard->persist();
        }
    }


    /**                                                            ]
     * Insere log de alteracao dos efetivos
     * @static
     * @param integer     $nrorg              Numero da organizacao
     * @param integer     $cdfilial           Codigo da filial
     * @param integer     $cdservico          Codigo do servico
     * @param integer     $dtcardapio         Data do cardapio
     * @param integer     $qtd_efetivos_ant   Qtd de efetivos anterior
     * @param integer     $qtd_efetivos_atual Qtd de efetivos atual
     */
    private static function insereLogAlteracaoEfetivos($nrorg, $cdfilial, $cdservico, $dtcardapio, $qtd_efetivos_ant, $qtd_efetivos_atual, $operador)
    {
        $dtCardapio = new \TkDateTime($dtcardapio);
        $dtProcessamento = new \TkDateTime(\ParametrosEstoque::getDataProcessamento());
        if ($dtCardapio->isEarlier($dtProcessamento) && $qtd_efetivos_ant != null) {
            $dtcardapio = new \DateTime($dtcardapio);
            /** @var $logAltEfet Logaltefet */
            $logAltEfet = new Logaltefet();
            $logAltEfet->setNrorg($nrorg);
            $logAltEfet->setCdfilial($cdfilial);
            $logAltEfet->setCdservfili($cdservico);
            $logAltEfet->setDtlogaltefet(DateUtil::getDataAtual());
            $logAltEfet->setQtefetant($qtd_efetivos_ant);
            $logAltEfet->setQtefetatual($qtd_efetivos_atual);
            $logAltEfet->setCdoperador($operador);
            $logAltEfet->setIdatualrop('N');
            $logAltEfet->setDtaltefetivo($dtcardapio);
            $logAltEfet->persist();
        }

    }


    public static function retornaLabelTab($servico, $dtCard){
        /** @var $dadosServico Servico */
        $dadosServico = Servico::getObjectByWhere(
            'NRORG = ? AND CDSERVFILI = ?',
            array(Ambiente::getNrorg(), $servico)
        );

        $nmServico = $dadosServico->getNmservico();

        $label = "Efetivos - $servico: $nmServico | $dtCard";
        return $label;
    }


}